#include <math.h>
#include "ai.h"
#include "map.h"
#include "playerpool.h"
#include "unit.h"
#include "structure.h"
#include "unitorstructure.h"
#include "weaponspool.h"

using namespace std;

AI::AI(CnCMap *theMap,
       Player *thePlayer) {
    myMap = theMap;
    myUASP = myMap->getUASP();
    myPlayerPool = myMap->getPlayerPool();
    myPlayer = thePlayer;
    myMapWidth = myMap->getWidth();
    myAttackType = DEFAULT_ATTACK;
}

void AI::printAttackType() {
    if ((myAttackType & ATTACK_WITH_ALL) == ATTACK_WITH_ALL) {
        cerr << "  do attack with all" << endl;
    } else {
        cerr << "  do not attack with all" << endl;
    }
    if ((myAttackType & ATTACK_ONE_ON_ONE) == ATTACK_ONE_ON_ONE) {
        cerr << "  do attack one on one" << endl;
    } else {
        cerr << "  do not attack one on one" << endl;
    }
    if ((myAttackType & ATTACK_NEAREST) == ATTACK_NEAREST) {
        cerr << "  do attack nearest" << endl;
    } else {
        cerr << "  do not attack nearest" << endl;
    }
    if ((myAttackType & ATTACK_MATCHED_TYPE) == ATTACK_MATCHED_TYPE) {
        cerr << "  do attack matched type" << endl;
    } else {
        cerr << "  do not attack matched type" << endl;
    }
}

void AI::toggleAttackType(Uint16 theAttackType) {
    myAttackType ^= theAttackType;
    printAttackType();
}

void AI::attackOtherSide() {
    Player* tmp;
    PSIDE aTargetSide = myPlayer->getOtherSide();

    // get all attacking units
    vector<UnitOrStructure *> aAttackingUOSs(0);
    vector<Unit *> aMyUnits = myPlayer->getUnits();
    vector<Structure*> aMyStructures = myPlayer->getStructures();
    // all attacking units
    for (vector<Unit *>::iterator aUnitsI = aMyUnits.begin();
            aUnitsI != aMyUnits.end();
            aUnitsI++) {
        if ((*aUnitsI)->isAlive() && (*aUnitsI)->canAttack()) {
            aAttackingUOSs.push_back(*aUnitsI);//pander store location
        }
    }
    for (vector<Structure*>::iterator aStructsI = aMyStructures.begin();
            aStructsI != aMyStructures.end();
            ++aStructsI) {
        if ((*aStructsI)->isAlive() && (*aStructsI)->canAttack()) {
            aAttackingUOSs.push_back(*aStructsI);
        }
    }
    if (PDBG < 3)
        cerr << "Attacking units and structures: " << aAttackingUOSs.size() << endl;

    // get all units or structures that can be attacked
    vector<UnitOrStructure *> aTargetUOSs(0);
    vector<Uint8> aTargetOwners = myPlayerPool->getOwnersForSide(aTargetSide);
    // all target owners
    for (vector<Uint8>::iterator aTargetOwnersI = aTargetOwners.begin();
            aTargetOwnersI != aTargetOwners.end();
            aTargetOwnersI++) {
        tmp = myPlayerPool->getPlayer(*aTargetOwnersI);
        vector<Unit *> aUnits = tmp->getUnits();
        // all target units
        for (vector<Unit *>::iterator aUnitsI = aUnits.begin();
                aUnitsI != aUnits.end();
                aUnitsI++) {
            if ((*aUnitsI)->isAlive()) {
                aTargetUOSs.push_back(*aUnitsI);
            }
        }
        vector<Structure *> aStructures = tmp->getStructures();
        // all target structures
        for (vector<Structure *>::iterator aStructuresI = aStructures.begin();
                aStructuresI != aStructures.end();
                aStructuresI++) {
            if ((*aStructuresI)->isAlive()) {
                aTargetUOSs.push_back(*aStructuresI);
            }
        }
    }
    if (PDBG < 3)
        cerr << "Targets units or structures: " << aTargetUOSs.size() << endl;

    // determine which unit attacks which
    // all attacking units
    for (vector<UnitOrStructure *>::iterator aAttackingUOSsI = aAttackingUOSs.begin();
            aAttackingUOSsI != aAttackingUOSs.end();
            aAttackingUOSsI++) {
        // determine coordinates
        float aMinUrgency = myMapWidth * myMapWidth * myMapWidth * myMapWidth;
        //    Uint16 aNTUx, aNTUy;
        vector<UnitOrStructure *>::iterator aSuitableTargetUOS = 0;

        // all target units
        for (vector<UnitOrStructure *>::iterator aTargetUOSsI = aTargetUOSs.begin();
                aTargetUOSsI != aTargetUOSs.end();
                aTargetUOSsI++) {
            float aUrgency = calcUrgency(*aAttackingUOSsI,
                                         *aTargetUOSsI);
            if (aUrgency == 1000.0)
                continue;
            if (aUrgency < aMinUrgency) {
                // get urgent unit
                aSuitableTargetUOS = aTargetUOSsI;
                aMinUrgency = aUrgency;
            }
        }
        // attack urgent unit
        if (aSuitableTargetUOS) {
            (*aAttackingUOSsI)->attackUnitOrStructure(*aSuitableTargetUOS);
            //(*aAttackingUOSsI)->referTo();
            //(*aSuitableTargetUOS)->referTo();

            if (PDBG < 3)
                cerr << "  Actual Target unit or structure: speed="
                << (int)((UnitType *)(*aSuitableTargetUOS)->getType())->getSpeed()
                << " rotspeed="
                << (int)((UnitType *)(*aSuitableTargetUOS)->getType())->getROT()
                << " inf="
                << (*aSuitableTargetUOS)->getType()->isInfantry()
                << " diff="
                << aMinUrgency
                << endl;
            //		      << " x=" << aNTUx
            //		      << " y=" << aNTUy << endl;
            if ((myAttackType & ATTACK_ONE_ON_ONE) == ATTACK_ONE_ON_ONE) {
                aTargetUOSs.erase(aSuitableTargetUOS);
            }
        }
    }
}

float AI::calcUrgency(UnitOrStructure * const theAUOS,
                      UnitOrStructure * const theTUOS) {
    //pander Uint16 aASubpos = theAUOS->getSubpos();
    Uint16 aAPos = theAUOS->getPos();
    Uint16 aTPos = theTUOS->getPos();
    Uint16 aAx = aAPos%myMapWidth;
    Uint16 aAy = aAPos/myMapWidth;
    Uint16 aTx = aTPos%myMapWidth;
    Uint16 aTy = aTPos/myMapWidth;
    //pander    bool aAW = theAUOS->getType()->getWeapon(true);
    bool aTWeapon = theTUOS->getType()->getWeapon(true);
    bool aAInfantry = theAUOS->getType()->isInfantry();
    bool aTInfantry = theTUOS->getType()->isInfantry();
    //    Uint16 aASpeed = theAUOS->getType()->getSpeed();
    //    Uint16 aTSpeed = theTUOS->getType()->getSpeed();
    float aDist = 0;
    if ((myAttackType & ATTACK_NEAREST) == ATTACK_NEAREST) {
        aDist = sqrt((aAx - aTx) * (aAx - aTx) +
                     (aAy - aTy) * (aAy - aTy));
    }
    if (theAUOS->getType()->getSpeed() == 0) {
        // structures can't move, so this target can be discarded
        if (aDist > theAUOS->getType()->getWeapon(true)->getRange())
            return 1000.0;
    }
    float aSimil = 1.00;
    if ((myAttackType & ATTACK_MATCHED_TYPE) == ATTACK_MATCHED_TYPE) {
        if (!aTWeapon) {
            aSimil *= 2.0; // attack unit or structure without weapons
        }
        if (aAInfantry) {
            if (aTInfantry) {
                aSimil *= 1.0; // infantry attacks infantry
            }
            else {
                aSimil *= 1.4; // infantry attacks vehicle
            }
        }
        else {
            if (aTInfantry) {
                aSimil *= 1.2; // vehicle attacks infantry
            }
            else {
                aSimil *= 1.0; // vehicle attacks vehicle
            }
        }
    }
    if (PDBG < 2)
        cerr << "dist: " << aDist << " simil: " << aSimil << endl;
    return aDist * aSimil;
    //pander	    theSpeed * theSpeed);
}

void AI::lostUnit(Unit* theUnit) {}

void AI::lostStructure(Structure* theStructure) {}

void AI::lostUnitOrStructure(UnitOrStructure* theUnitOrStructure) {}
