#ifndef __STRUCTURE_H
#define __STRUCTURE_H

#ifdef _MSC_VER
#pragma warning (disable: 4503)
#pragma warning (disable: 4786)
#endif

class Structure;
class StructureType;

#include <vector>
#include "SDL.h"
#include "shpimage.h"
#include "inifile.h"
#include "actioneventqueue.h"
#include "unitandstructurepool.h"
#include "unitorstructure.h"
#include "playerpool.h"
#include "ai.h"

typedef struct {
    Uint32 animdelay;
    Uint8 loopend, loopend2, animspeed, animtype, sectype;
    Uint8 dmgoff, dmgoff2;
    Uint16 makenum;
}
animinfo_t;

typedef struct {
    Uint16 power;
    Uint16 drain;
    bool powered;
}
powerinfo_t;

class Weapon;
class WeaponsPool;
class Unit;

class StructureType : public UnitOrStructureType {
public:
    StructureType(std::vector<SHPImage *> *imgPool, char *tname, INIFile *structini,
                  char *thext,WeaponsPool* wpool);
    ~StructureType();
    Uint16 *getSHPNums() {
        return shpnums;
    }
    Uint16 *getSHPTNum() {
        return shptnum;
    }

    Uint8 getNumLayers() {
        return numshps;
    }

    Uint16 getMakeImg() {
        return makeimg;
    }

    bool isWall() {
        return is_wall;
    }

    Uint8 getXsize() {
        return xsize;
    }
    Uint8 getYsize() {
        return ysize;
    }

    Uint8 isBlocked(Uint16 tile) {
        return blocked[tile];
    }

    Sint16 getXoffset() {
        return xoffset;
    }
    Sint16 getYoffset() {
        return yoffset;
    }
    Uint16 getCost() {
        return cost;
    }
    Uint8 getTurnspeed() {
        return turnspeed;
    }
    Uint8 getSight() {
        return sight;
    }
    Uint8 getSpeed() {
        return 0;
    }
    Uint8 getArmour() {
        return armour;
    }
    animinfo_t getAnimInfo() {
        return animinfo;
    }

    powerinfo_t getPowerInfo() {
        return powerinfo;
    }
    Uint16 getMaxHealth() {
        return maxhealth;
    }
    Weapon *getWeapon(bool primary = true) {
        return (primary?primary_weapon:secondary_weapon);
    }
    bool hasTurret() {
        return turret;
    }
    Uint16 getBlckOff() {
        return blckoff;
    }
    bool isInfantry() {
        return false;
    }
    Uint8 getNumWallLevels() {
        return numwalllevels;
    }
    Uint8 getDefaultFace() {
        return defaultface;
    }
private:
    Uint16 *shpnums, *shptnum;
    Uint16 cost,maxhealth,makeimg,blckoff;
    Sint16 xoffset,yoffset;
    Uint8 armour,turnspeed,sight,xsize,ysize,numshps,numwalllevels,defaultface;
    Uint8 *blocked;

    Weapon* primary_weapon;
    Weapon* secondary_weapon;

    animinfo_t animinfo;
    powerinfo_t powerinfo;

    bool is_wall,turret;
};

class BAttackAnimEvent;

class Structure : public UnitOrStructure {
public:
    friend class BuildingAnimEvent;
    friend class BAttackAnimEvent;

    Structure(StructureType *type, Uint16 cellpos, Uint8 owner);
    ~Structure();
    Uint8 getImageNums(Uint32 **inums);
    Uint16* getImageNums() {
        return imagenumbers;
    }
    void changeImage(Uint8 layer, Sint16 imagechange) {
        imagenumbers[layer]+=imagechange;
    }
    Uint32 getImageNum(Uint8 layer) {
        return type->getSHPNums()[layer]+imagenumbers[layer];
    }
    UnitOrStructureType *getType() {
        return type;
    }
    void select() {
        selected = true;
    }
    void unSelect() {
        selected = false;
    }
    bool isSelected() {
        return selected;
    }
    void setStructnum(Uint16 stn) {
        structnum = stn;
    }
    Uint16 getStructnum() {
        return structnum;
    }
    Uint16 getPos() {
        return cellpos;
    }
    Uint16 getBPos() {
        return bcellpos;
    }
    void referTo() {
        references++;
    }
    void unrefer();
    void remove();
    bool isAlive() {
        return !deleted;
    }
    static void setUASP(UnitAndStructurePool *usp) {
        uspool = usp;
    }
    Uint16 getSubpos() {
        return 0;
    }
    void applyDamage(Sint16 amount, Weapon* weap);
    void runAnim(Uint32 mode);
    void runSecAnim(Uint32 param);
    void stopAnim();
    Uint8 getOwner() {
        return owner;
    }
    void setOwner(Uint8 newowner) {
        owner = newowner;
    }
    bool canAttack() {
        return type->getWeapon()!=NULL;
    }
    void attackUnit(Unit* un);
    void attackStructure(Structure* st);
    void attackUnitOrStructure(UnitOrStructure* uos);
    Uint16 getHealth() {
        return health;
    }
    Sint16 getXoffset() {
        return type->getXoffset();
    }
    Sint16 getYoffset() {
        return type->getYoffset();
    }
    bool isWall() {
        return type->isWall();
    }
    double getRatio() {
        return ratio;
    }
private:
    Uint8 checkdamage();
    void setimage(Uint16 frame) {
        setimage(frame,0);
    }
    void setimage(Uint16 frame,Uint8 layer) {
        imagenumbers[layer]=(frame)|(imagenumbers[layer]&0x800);
    }
    StructureType *type;
    Uint16 *imagenumbers;
    Uint16 cellpos,bcellpos,health,structnum;
    Uint8 owner,references,damaged;
    bool deleted,selected,animating,usemakeimgs,exploding;
    double ratio; // health/maxhealth

    static UnitAndStructurePool* uspool;
    BuildingAnimEvent* buildAnim;
    BAttackAnimEvent* attackAnim;
};

#include "structureanims.h"

#endif
