#ifndef __UNIT_H
#define __UNIT_H

#ifdef _MSC_VER
#pragma warning (disable: 4503)
#pragma warning (disable: 4786)
#endif

#define MOVE_UP 0
#define MOVE_UPRIGHT 1
#define MOVE_RIGHT 2
#define MOVE_DOWNRIGHT 3
#define MOVE_DOWN 4
#define MOVE_DOWNLEFT 5
#define MOVE_LEFT 6
#define MOVE_UPLEFT 7


#include <vector>
#include "SDL.h"
#include "shpimage.h"
#include "inifile.h"
#include "actioneventqueue.h"

class InfantryGroup;
class UnitType;
class Unit;
//class MoveAnimEvent;
#include "unitandstructurepool.h"
#include "unitanimations.h"
#include "path.h"
#include "unitorstructure.h"
#include "memtrack.h"
#include "playerpool.h"
#include "ai.h"

class Weapon;
class WeaponsPool;
class StructureType;

class UnitType : public UnitOrStructureType {
public:
    UnitType(std::vector<SHPImage *> *imgPool, char *tname, INIFile *unitini,
             WeaponsPool *wpool, UnitAndStructurePool* uspool);
    ~UnitType();
    Uint32 *getSHPNums() {
        return shpnums;
    }
    Uint8 getNumLayers() {
        return numlayers;
    }
    bool isInfantry() {
        return is_infantry;
    }
    //   Uint8 getSize(){return size;}
    Uint8 getOffset() {
        return offset;
    }
    Uint8 getSpeed() {
        return speed;
    }
    Uint8 getROT() {
        return turnspeed;
    }
    Sint8 getMoveMod() {
        return movemod;
    }
    Uint8 getTurnMod() {
        return turnmod;
    }
    Uint16 getMaxHealth() {
        return maxhealth;
    }
    Uint16 getCost() {
        return cost;
    }
    Uint8 getTurnspeed() {
        return turnspeed;
    }
    Uint8 getSight() {
        return sight;
    }
    Uint8 getArmour() {
        return armour;
    }
    char* getRandTalk(Uint8 type);
    Weapon *getWeapon(bool primary = true) {
        return (primary?primary_weapon:secondary_weapon);
    }
    bool isWall() {
        return false;
    }
    bool canDeploy() {
        return deployable;
    }
    char* getDeployTarget() {
        return deploytarget;
    }
    StructureType* getDeployType() {
        return deploytype;
    }
private:
    Uint32 *shpnums;
    Uint16 cost,maxhealth;
    Uint8 numlayers,armour,speed,turnspeed,turnmod,sight,offset;
    Sint8 movemod;


    // Talkback related members
    void gettbvecptr(char** line, std::vector<char*>** vecptr);
    void createtalkback(char* talkback);
    std::vector<char*> tbReport;
    std::vector<char*> tbAck;
    std::vector<char*> tbDie;
    std::vector<char*> tbPKill;
    std::vector<char*> tbBAtk;

    bool is_infantry, deployable;
    char* deploytarget;
    // this is used to check the unit can deploy
    StructureType* deploytype;
    Uint8 maxpassengers;

    Weapon* primary_weapon;
    Weapon* secondary_weapon;
};


class Unit : public UnitOrStructure {
public:
    friend class UnitAnimEvent;
    friend class MoveAnimEvent;
    friend class UAttackAnimEvent;
    friend class TurnAnimEvent;
    Unit(UnitType *type, Uint16 cellpos, Uint8 subpos, InfantryGroup *group, Uint8 owner);
    ~Unit();
    Uint8 getImageNums(Uint32 **inums, Sint8 **xoffsets, Sint8 **yoffsets);
    static void setUASP(UnitAndStructurePool *usp) {
        uspool = usp;
    }
    InfantryGroup *getInfantryGroup() {
        return infgrp;
    }
    void setInfantryGroup(InfantryGroup *ig) {
        infgrp = ig;
    }
    Uint32 getImageNum(Uint8 layer) {
        return type->getSHPNums()[layer]+imagenumbers[layer];
    }
    Sint16 getXoffset() {
        return xoffset-type->getOffset();
    }
    Sint16 getYoffset() {
        return yoffset-type->getOffset();
    }
    UnitOrStructureType* getType() {
        return type;
    }
    void select() {
        selected = true;
    }
    void unSelect() {
        selected = false;
    }
    bool isSelected() {
        return selected;
    }
    Uint16 getPos() {
        return cellpos;
    }
    Uint16 getBPos() {
        return cellpos;
    }
    Uint16 getSubpos() {
        return subpos;
    }
    Uint16 getUnitnum() {
        return unitnum;
    }
    void setUnitnum(Uint16 unum) {
        unitnum = unum;
    }
    Uint16 getHealth() {
        return health;
    }

    //   void setPath();
    //   void move(Uint8 dir);
    void move(Uint16 dest);
    void attackUnit(Unit *un);
    void attackStructure(Structure *st);
    void attackUnitOrStructure(UnitOrStructure* uos);
    void stop();

    void setFace(Uint8 layer, Uint8 dir) {
        imagenumbers[layer]&=0xffe0;
        imagenumbers[layer] |= dir;
    }
    void setIFace(Uint8 layer, Uint8 face) {
        imagenumbers[layer] = face | (getImageNum(layer)&0x800);
    }
    Uint8 getOwner() {
        return owner;
    }
    void setOwner(Uint8 newowner) {
        owner = newowner;
    }
    void referTo() {
        references++;
    }
    void unrefer();
    void remove();
    bool isAlive() {
        return !deleted;
    }
    void applyDamage(Sint16 amount, Weapon* weap);
    bool canAttack() {
        return type->getWeapon()!=NULL;
    }
    void doRandTalk(Uint8 ttype);
    UnitAndStructurePool* getUSPool() {
        return uspool;
    }
    void deploy(void);
    bool canDeploy();
    bool checkDeployTarget(Uint32 pos);
    Uint32 calcDeployPos();
    double getRatio() {
        return ratio;
    }
private:
    UnitType *type;
    Uint16 *imagenumbers;
    Uint16 unitnum,cellpos,health;
    Uint8 owner,references,subpos,passengers;
    Sint8 xoffset,yoffset;
    bool deleted,selected,deployed,stopping;
    double ratio;

    InfantryGroup *infgrp;

    UnitAnimEvent* PriAnim, *SecAnim;
    UnitAnimEvent* TerAnim; // Tertiary animation
    // (used for turret, while turning while moving)
    /* the stop parameter determines whether the change over is (if stop is true)
     * 1 setschedule new event onto existing event
     * 2 stop() existing event (this means that the change over is not instant)
     * 3 the existing animevent schedules the new event as the existing animevent 
     *   destructs.
     * or (if stop is false)
     * 1 the pointer variable storing the existing anim event is instantly updated
     *   to point to the new event.
     * 2 the new event is then scheduled.
     * note that the prior animevent must be taken care of in the second case (either
     * it is scheduled to restart after the new event terminates or it deletes itself
     * and returns)
     */
    /* the first three functions serve as an interface to the fourth function (which
     * does all the work).
     * the first three functions are identical except setPriAnim passes a pointer to
     * the PriAnim pointer variable, setSecAnim a pointer to the SecAnim p. variable, 
     * etc.
     */
    void setPriAnim(UnitAnimEvent* e,bool stop);
    void setSecAnim(UnitAnimEvent* e,bool stop);
    void setTerAnim(UnitAnimEvent* e,bool stop);
    void setAnim(UnitAnimEvent* e, UnitAnimEvent** anim, bool stop);
    /* missing: order, target, temptarget and path */
    static UnitAndStructurePool *uspool;
};

/*
 * This should be a member of unit for infatry. When a infatry unit walks in
 * to a new cell a new group is created, if someone is already there that 
 * group is used. We need one more bit in the unit/structure matrix to tell if
 * a infantry is in that cell (faster). The groups should be destroyed 
 * (or reused by last unit if he travels to a empty cell) when the last
 * infantry leaves. */

class InfantryGroup {
public:
    InfantryGroup() {
        int i;
        numinfantry = 0;
        for(i=0;i<5;i++)
            positions[i] = NULL;
    }
    void addInfantry(Unit *inf, Uint8 subpos) {
        positions[subpos] = inf;
        numinfantry++;
    }
    void removeInfantry(Uint8 subpos) {
        positions[subpos]=NULL;
        numinfantry--;
    }
    bool isClear(Uint8 subpos) {
        return (positions[subpos] == NULL);
    }
    Uint8 getNumInfantry() {
        return numinfantry;
    }
    Unit *unitAt(Uint8 subpos) {
        return positions[subpos];
    }
    Uint8 getImageNums(Uint32 **inums, Sint8 **xoffsets, Sint8 **yoffsets) {
        int i, j;

        (*inums) = new Uint32[numinfantry];
        (*xoffsets) = new Sint8[numinfantry];
        (*yoffsets) = new Sint8[numinfantry];
        j = 0;
        for( i = 0; i < 5; i++ ) {
            if( positions[i] != NULL ) {
                (*inums)[j]=positions[i]->getImageNum(0);
                (*xoffsets)[j]=positions[i]->getXoffset()+unitoffset[i];
                (*yoffsets)[j]=positions[i]->getYoffset()+unitoffset[i+5];
                j++;
            }
        }
        return numinfantry;
    }
    void getSubposOffsets(Uint8 oldsp, Uint8 newsp, Sint8 &xoffs, Sint8 &yoffs) {
        xoffs = unitoffset[oldsp]-unitoffset[newsp];
        yoffs = unitoffset[oldsp+5]-unitoffset[newsp+5];
    }
    static const Sint8* getunitoffsets() {
        return unitoffset;
    }
private:
    Unit* positions[5];
    Uint8 numinfantry;
    static const Sint8 unitoffset[];
    /* Theses values have been heavily tested, do NOT change them unless you're
     _really_ sure of what you are doing */
    /* X value */
    //      -13, -19, -7, -19, -7,
    /* Y value */
    //	-3, -7, -7, 1, 1
    //   };
}
;

#endif


