#ifndef __UNITANDSTRUCTUREPOOL_H
#define __UNITANDSTRUCTUREPOOL_H




#ifdef _MSC_VER
#pragma warning (disable: 4503)
#pragma warning (disable: 4786)
#endif




#include <string>
#include <map>
#include <vector>

#include "SDL.h"

class UnitAndStructurePool;

#include "shpimage.h"
#include "structure.h"
#include "unit.h"
#include "structure.h"
#include "inifile.h"
#include "actioneventqueue.h"
#include "soundengine.h"
#include "map.h"
#include "unitorstructure.h"

#define US_LOWER_RIGHT  0x80000000
#define US_IS_UNIT      0x40000000
#define US_IS_STRUCTURE 0x20000000
#define US_IS_WALL      0x10000000

#define US_MOVING_HERE  0x07000000

#define US_HAS_L2OVERLAY 0x08000000
//#define US_HAS_PROJECTILE 0x00800000
//#define US_HAS_HIGHPROJ   0x00400000
//#define US_HAS_EXPLOTION  0x00200000

//#define US_HAS_AIRUNIT

#ifndef __GNUC__
int strcasecmp(char*, char*);
#endif

class WeaponsPool;

class L2Overlay {
public:
    Uint32 imagenum;
    Uint16 cellpos;
    Sint8 xoffset;
    Sint8 yoffset;
};

/// Stores all units and structures.
/// Handles level two overlays.
class UnitAndStructurePool {

public:

    UnitAndStructurePool(CnCMap *ccmap, ActionEventQueue *evQueue);
    ~UnitAndStructurePool();

    Uint8 getUnitOrStructureNum(Uint16 cellpos, Uint32 **inumbers, Sint8 **xoffsets, Sint8 **yoffsets);
    bool hasL2overlay(Uint16 cellpos) {
        return (unitandstructmat[cellpos]&US_HAS_L2OVERLAY)!=0;
    }
    Uint8 getL2overlays(Uint16 cellpos, Uint32 **inumbers, Sint8 **xoddset, Sint8 **yoffset);
    std::multimap<Uint16, L2Overlay*>::iterator addL2overlay(Uint16 cellpos, L2Overlay *ov);
    void removeL2overlay(std::multimap<Uint16, L2Overlay*>::iterator entry);

    void createStructure( char *typen, Uint16 cellpos, Uint8 owner );
    void createStructure( char *typen, Uint16 cellpos, Uint8 owner, bool makeanim );
    void createUnit( char *typen, Uint16 cellpos, Uint8 subpos, Uint8 owner );
    Unit *getUnitAt(Uint32 cell, Uint8 subcell);
    Structure *getStructureAt(Uint32 cell);
    Structure* getStructureAt(Uint32 cell, bool wall);
    UnitOrStructure* getUnitOrStructureAt(Uint32 cell, Uint8 subcell);
    Uint16 getSelected(Uint32 pos);
    Uint16 preMove(Unit *un, Uint8 dir, Sint8 *xmod, Sint8 *ymod);
    Uint8 postMove(Unit *un, Uint16 newpos);
    void abortMove(Unit* un, Uint32 pos);
    void scheduleUSEvent(ActionEvent *ev) {
        evQueue->scheduleEvent(ev);
    }
    UnitType* getUnitTypeByName(char* unitname);
    StructureType *getStructureTypeByName(char *structname);
    bool freeTile(Uint16 pos) {
        return (unitandstructmat[pos]&0x70000000)==0;
    }
    Uint16 getTileCost(Uint16 pos);
    Uint16 getTileCost(Uint16 pos, Unit* excpUn);
    CnCMap *getMap() {
        return ccmap;
    }
    void setCostCalcOwnerAndType(Uint8 owner, Uint8 type) {
        costcalcowner = owner;
        costcalctype = type;
    }
    void removeUnit(Unit *un);
    void removeStructure(Structure *st);
    bool hasDeleted() {
        bool retval = deleted_unitorstruct;
        deleted_unitorstruct = false;
        return retval;
    }
    void setsfxeng(SoundEngine* seng) {
        sfxeng = seng;
    }
    SoundEngine* getsfxeng() {
        return sfxeng;
    }
    WeaponsPool *getWeaponsPool() {
        return wp;
    }
    void showMoves();
private:

    CnCMap *ccmap;
    char theaterext[5];

    Uint32 *unitandstructmat;

    std::vector<Structure *> structurepool;
    std::vector<StructureType *> structuretypepool;
    std::map<std::string, Uint16> structname2typenum;

    std::vector<Unit *> unitpool;
    std::vector<UnitType *> unittypepool;
    std::map<std::string, Uint16> unitname2typenum;

    std::multimap<Uint16, L2Overlay*> l2pool;

    INIFile *structini;
    INIFile *unitini;

    ActionEventQueue *evQueue;
    SoundEngine* sfxeng;
    Uint8 costcalcowner;
    Uint8 costcalctype;

    WeaponsPool *wp;
    bool deleted_unitorstruct;
    Uint16 numdeletedunit;
    Uint16 numdeletedstruct;
    void updateWalls(Structure* st, bool add);
};



#endif
