#ifndef __UNITANIMATIONS_H
#define __UNITANIMATIONS_H


#include "SDL.h"
#include "path.h"

class Unit;
class UnitOrStructure;
class UnitAnimEvent;

typedef struct u_animinfo_t {
    Uint8 type;
    char* defaultimage;
    Uint16** frames;
    char** specific_images;
    Uint8 length;
    char* sound;
    Uint8 location;
    Uint8 prone_offset;
    u_animinfo_t* secanim_info;
};

class UnitAnimEvent : public ActionEvent {
private:
    Unit* un;
    UnitAnimEvent* scheduled;
    Uint8 mode;
protected:
    u_animinfo_t parseAnimEntry(char* name);
public:
    UnitAnimEvent(Uint32 p, Unit* un, Uint8 smode);
    virtual ~UnitAnimEvent();
    void setSchedule(UnitAnimEvent* e);
    virtual void stop() = 0;
    virtual Uint8 getMode() {
        return mode;
    }
    virtual void run() = 0;
    virtual void setRange(Uint32 nr);
    virtual void changedir(Uint8 dir);
};

/* Common code that can't be inherited w/o using protected data
class GenericAnimEvent : public UnitAnimEvent {
private:
	Uint8 mode;
	bool stopping;
	u_animinfo_t animinfo;
public:
	GenericAnimEvent(Uint32 p, Unit* un, ...);
	virtual ~GenericAnimEvent();
	virtual Uint8 getMode() {return mode;}
	virtual void stop() {stopping = true;}
	virtual void run();
};
*/

class MoveAnimEvent : public UnitAnimEvent {
private:
    Uint8 mode;
    bool stopping;
    void startMoveOne(bool wasblocked);
    void moveDone();
    Uint16 dest,newpos;
    bool blocked, moved_half, walking;
    Sint8 xmod, ymod;
    Unit* un;
    Path* path;
    Uint8 istep,dir;
    Uint32 range;
public:
    MoveAnimEvent(Uint32 p, Unit* un, Uint16 dest);
    virtual ~MoveAnimEvent();
    virtual Uint8 getMode() {
        return mode;
    }
    virtual void stop();
    virtual void run();
    virtual void setRange(Uint32 nr) {
        range = nr;
    }
};

class WalkAnimEvent : public UnitAnimEvent {
private:
    Uint8 mode;
    bool stopping;
    void calcbaseimage(void);
    Unit* un;
    Uint8 dir, istep, layer, baseimage;
public:
    WalkAnimEvent(Uint32 p, Unit* un, Uint8 dir, Uint8 layer);
    virtual ~WalkAnimEvent();
    virtual Uint8 getMode() {
        return mode;
    }
    virtual void stop() {
        stopping = true;
    }
    virtual void run();
    virtual void changedir(Uint8 ndir) {
        dir = ndir;
        calcbaseimage();
    }
};

class TurretAnimEvent: public UnitAnimEvent {
private:
    Uint8 mode;
    bool stopping;
    Sint8 turnmod;
    Uint8 dir,layer;
    Unit* un;
public:
    TurretAnimEvent(Uint32 p, Unit *un, Uint8 dir, Uint8 layer, UnitAnimEvent *e, Uint8 smode);
    virtual ~TurretAnimEvent();
    virtual void run();
    virtual Uint8 getMode() {
        return mode;
    }
    virtual void stop() {
        stopping = true;
    }
};

class TurnAnimEvent : public UnitAnimEvent {
private:
    Uint8 mode;
    bool stopping,runonce;
    Sint8 turnmod;
    Unit *un;
    Uint8 dir;
    Uint8 layer;
public:
    TurnAnimEvent(Uint32 p, Unit *un, Uint8 dir, Uint8 layer, UnitAnimEvent *e,Uint8 smode);
    virtual ~TurnAnimEvent();
    virtual void run();
    virtual Uint8 getMode() {
        return mode;
    }
    virtual void stop() {
        stopping = true;
    }
};

class UAttackAnimEvent : public UnitAnimEvent {
private:
    Unit *un;
    Uint8 mode;
    bool stopping;
    UnitOrStructure* target;
public:
    UAttackAnimEvent(Uint32 p, Unit *un, UnitOrStructure *target);
    virtual ~UAttackAnimEvent();
    void stop();
    virtual Uint8 getMode() {
        return mode;
    }
    virtual void run();
};

#endif /*__UNITANIMATIONS_H*/
