#ifndef __VFS_H
#define __VFS_H

#include "SDL.h"

/* Virtual file class.
 * All virtual files are opened for reading only since it's supposed to be 
 * used for graphics, sound and other data loading to games.
 */
class VFile {
public:
    VFile(FILE *file_to_load, Uint32 startpos, Uint32 filesize, bool externalFile);
    ~VFile();
    Uint32 readByte(Uint8 *databuf, Uint32 numBytes);
    Uint32 readWord(Uint16 *databuf, Uint32 numWords);
    Uint32 readThree(Uint32 *databuf, Uint32 numThrees);
    Uint32 readDWord(Uint32 *databuf, Uint32 numDwords);
    char *getLine(char *string, Uint32 buflen);
    void seekSet(Uint32 pos) {
        this->pos=pos;
        if(this->pos>size) {
            this->pos=size;
        }
    }
    void seekCur(Sint32 offset) {
        pos+=offset;
        if(pos>size) {
            pos=size;
        }
    }
    Uint32 fileSize() {
        return size;
    }
    Uint32 tell() {
        return pos;
    }
private:
    FILE *datafile;
    Uint32 startpos;
    Uint32 pos;
    Uint32 size;
    bool externalFile;
};

extern void VFS_Init();
extern void VFS_Destroy();
extern VFile *VFS_Open(char *fname);
extern void VFS_Close(VFile *file);

#endif
