#if defined _DEBUG && !defined _WIN32
#include <stdio.h>
#include <list>

using namespace std;

typedef struct {
    unsigned int address;
    unsigned int size;
    char file[64];
    unsigned int line;
    bool array;
}
ALLOC_INFO;

typedef list<ALLOC_INFO*> AllocList;

AllocList allocList;

void OutputDebugString(char *msg) {
    printf("%s", msg);
}


void AddTrack(unsigned int addr, unsigned int asize,  const char *fname,
              unsigned int lnum, bool array) {
    ALLOC_INFO *info;
    //   if(!allocList) {
    //      allocList = new AllocList();
    //   }

    //   info = new(ALLOC_INFO);
    info = (ALLOC_INFO *)malloc(sizeof(ALLOC_INFO));
    info->address = addr;
    strncpy(info->file, fname, 63);
    info->line = lnum;
    info->size = asize;
    info->array = array;
    allocList.insert(allocList.begin(), info);
    //fprintf(stderr,"AddTrack at %-50s:\tLine: %d,\tAdd: %d (%s)\n",
    //    fname,lnum,addr,(array?"array":"notarray"));
}

void RemoveTrack(unsigned int addr, bool array) {
    AllocList::iterator i;
    char buf[1024];

    //   if(!allocList)
    //     return;
    for(i = allocList.begin(); i != allocList.end(); i++) {
        if((*i)->address == addr) {
            if( (*i)->array != array ) {
                //	   printf("WARNING: missmatching [] in new/delete for:");
                sprintf(buf, "%-50s:\t\tLINE %d,\t\tADDRESS %d\thas missmatching new/delete ([])\n",
                        (*i)->file, (*i)->line, (*i)->address);
                OutputDebugString(buf);
            }
            //fprintf(stderr,"RemoveTrack at %-47s:\tLine: %d,\tAdd: %d (%s)\n",
            //    (*i)->file, (*i)->line, (*i)->address, ((*i)->array?"array":"notarray"));
            allocList.remove((*i));
            //	 free(*i);
            break;
        }
    }
    //#ifndef _MSC_VER
    if( i == allocList.end() ) {
        printf("WARNING: Trying to free unallocated memory\n");
#ifdef STRICT_DEBUG

        abort();
#endif

    }
    //#endif
}

void DumpUnfreed() {
    AllocList::iterator i;
    unsigned int totalSize = 0;
    char buf[1024];

    //   if(!allocList)
    //   return;

    for(i = allocList.begin(); i != allocList.end(); i++) {
        sprintf(buf, "%-50s:\t\tLINE %d,\t\tADDRESS %d\t%d unfreed\n",
                (*i)->file, (*i)->line, (*i)->address, (*i)->size);
        OutputDebugString(buf);
        totalSize += (*i)->size;
        free(*i);
    }
    sprintf(buf, "-----------------------------------------------------------\n");
    OutputDebugString(buf);
    sprintf(buf, "Total Unfreed: %d bytes\n", totalSize);
    OutputDebugString(buf);
}

//#ifdef _DEBUG
void * operator new(unsigned int size, const char *file,
                    int line) {
    void *ptr = (void *)malloc(size);
    //   printf("new\n");
    AddTrack((unsigned int)ptr, size, file, line, false);
    return(ptr);
};
void operator delete(void *p) {
    //	printf("del in %s at %d\n", file, line);
    RemoveTrack((unsigned int)p, false);
    free(p);
};
/*
void * operator new[](unsigned int size)
{
   void *ptr = (void *)malloc(size);
//   printf("wrong new[]\n");
   AddTrack((unsigned int)ptr, size, "unknown", 0, true);
   return(ptr);
};*/
void * operator new[](unsigned int size, const char *file,
                      int line) {
    void *ptr = (void *)malloc(size);
    //printf("new[]\n");
    AddTrack((unsigned int)ptr, size, file, line, true);
    return(ptr);
};
void operator delete[](void *p) {
    //	printf("del[] in %s at %d\n", file, line);
    RemoveTrack((unsigned int)p, true);
    free(p);
};

#endif
