#include "SDL.h"
#include <stdlib.h>
#include <string.h>

#include "cursor.h"

#include "game.h"

#include "memtrack.h"

#include "cursorpool.h"

/** Constructor, load the cursorimage and set upp all surfaces to buffer
 * background in etc.
 */
Cursor::Cursor() {
    SDL_Surface *tmp;

    curimg = 0;
    nimgs = 1;

    cursorimg = new Dune2Image("mouse.shp", -1);
    transicn = new TemplateImage("trans.icn", Game::mapscaleq);

    image[curimg] = cursorimg->getImage(0);

    /* All cursors loaded */

    tmp = SDL_CreateRGBSurface (SDL_SWSURFACE, image[0]->w, image[0]->h, 16, 0xff, 0xff, 0xff, 0);
    bg = SDL_DisplayFormat( tmp );
    SDL_FreeSurface( tmp );

    currentcursor = CUR_STANDARD;
    x = 0;
    y = 0;
    cursor_offset = 0;

    cursorpool = new CursorPool("cursors.ini");
    ci = NULL;
}

/** Destructor, free the surfaces */
Cursor::~Cursor() {
    int i;
    for( i = 0; i < nimgs; i++ )
        SDL_FreeSurface( image[i] );
    SDL_FreeSurface( bg );

    delete cursorimg;
    delete transicn;
    delete cursorpool;
}

/** Change active cursor.
 * @param number of the new cursor.
 */
void Cursor::setCursor( Uint16 cursornum, Uint8 animimages ) {
    int i;
    if( currentcursor == cursornum )
        return;

    for( i = 0; i < nimgs; i++ )
        SDL_FreeSurface( image[i] );

    curimg = 0;
    nimgs = animimages;
    for( i = 0; i < nimgs; i++ )
        image[i] = cursorimg->getImage(cursornum+i);

    if( cursornum != CUR_STANDARD )
        cursor_offset = -((image[0]->w)>>1);
    else
        cursor_offset = 0;

    currentcursor = cursornum;
}

void Cursor::setCursor(char* curname) {
    int i;

    ci = cursorpool->getCursorByName(curname);

    if( currentcursor == ci->anstart)
        return;

    for( i = 0; i < nimgs; i++ )
        SDL_FreeSurface( image[i] );

    curimg = 0;
    nimgs = ci->anend - ci->anstart;
    for( i = 0; i < nimgs; i++ )
        image[i] = cursorimg->getImage((ci->anstart)+i);

    if( strcasecmp(curname,"STANDARD") == 0 ) {
        cursor_offset = 0;
    } else {
        cursor_offset = -((image[0]->w)>>1);
    }

    currentcursor = ci->anstart;
}

void Cursor::setPlaceCursor(Uint8 stw, Uint8 sth, Uint8 *icn) {
    int i, x, y;
    SDL_Surface *transr, *transw, *bigimg;

    Uint8 *data;
    SDL_Rect dest;
    static Uint32 oldptr;
    Uint32 newptr;

    newptr = 0;
    for(i = 0; i < stw*sth; i++) {
        newptr = newptr<<1|(icn[i]==2?1:0);
    }

    if( currentcursor == CUR_PLACE && newptr == oldptr ) {
        return;
    }
    oldptr = newptr;

    for( i = 0; i < nimgs; i++ )
        SDL_FreeSurface( image[i] );
    curimg = 0;
    currentcursor = CUR_PLACE;
    nimgs = 1;
    cursor_offset = 0;

    transw = transicn->getImage(0);
    transr = transicn->getImage(2);

    data = new Uint8[stw*sth*transw->w*transw->h];
    memset(data, 0, stw*sth*transw->w*transw->h);

    //	image[0] = transicn->getImage(0);
    bigimg = SDL_CreateRGBSurfaceFrom(data, stw*transw->w, sth*transw->h,
                                      8, stw*transw->w, 0, 0, 0, 0);

    SDL_SetColors(bigimg, SHPBase::getPalette(0), 0, 256);
    SDL_SetColorKey(bigimg, SDL_SRCCOLORKEY, 0);

    image[0] = SDL_DisplayFormat(bigimg);
    SDL_FreeSurface(bigimg);
    delete[] data;

    dest.w = transw->w;
    dest.h = transw->h;

    dest.y = 0;
    for( y = 0; y < sth; y++ ) {
        dest.x = 0;
        for( x = 0; x < stw; x++ ) {
            if( icn[y*stw+x] == 1 ) {
                SDL_BlitSurface(transw, NULL, image[0], &dest);
            } else if( icn[y*stw+x] == 2 ) {
                SDL_BlitSurface(transr, NULL, image[0], &dest);
            }
            dest.x += dest.w;
        }
        dest.y += dest.h;
    }


    SDL_FreeSurface(transw);
    SDL_FreeSurface(transr);

}

SDL_Surface *Cursor::getCursor() {
    static Uint32 lastchange = 0;
    Uint32 tick = SDL_GetTicks();
    if( tick > lastchange +100 ) {
        curimg++;
        if( curimg >= nimgs )
            curimg = 0;
        lastchange = tick;
    }
    return image[curimg];
}
