#include <stdlib.h>
#include "selection.h"
#include "path.h"
#include "weaponspool.h"
#include "memtrack.h"

using namespace std;

Selection::Selection() {
    numinfantry = 0;
    numattacking = 0;
    enemies_selected = false;
}

Selection::~Selection() {
    sel_units.resize(0);
    sel_structures.resize(0);
}

void Selection::addUnit(Unit *selunit, bool enemy) {
    selunit->select();
    selunit->referTo();
    sel_units.push_back(selunit);
    if( ((UnitType *)selunit->getType())->isInfantry() )
        numinfantry++;

    enemies_selected = enemy;
    if( !enemy && selunit->canAttack() ) {
        numattacking++;
    }
}

void Selection::removeUnit(Unit *selunit) {
    Uint32 i;
    bool removing = false;

    selunit->unSelect();
    selunit->unrefer();

    for( i = 0; i < sel_units.size(); i++ ) {
        if( removing )
            sel_units[i-1] = sel_units[i];
        if( sel_units[i] == selunit )
            removing = true;
    }
    sel_units.resize(sel_units.size()-1);
    if( ((UnitType *)selunit->getType())->isInfantry() )
        numinfantry--;

    if( enemies_selected ) {
        enemies_selected = false;
    } else if( selunit->canAttack() ) {
        numattacking--;
    }
}

void Selection::addStructure(Structure *selstruct, bool enemy) {
    selstruct->select();
    selstruct->referTo();
    sel_structures.push_back(selstruct);
    enemies_selected = enemy;
    if( !enemy && selstruct->canAttack() ) {
        ++numattacking;
    }
}

void Selection::removeStructure(Structure *selstruct) {
    Uint32 i;
    bool removing = false;

    selstruct->unSelect();
    selstruct->unrefer();

    for( i = 0; i < sel_structures.size(); i++ ) {
        if( removing )
            sel_structures[i-1] = sel_structures[i];
        if( sel_structures[i] == selstruct )
            removing = true;
    }
    sel_structures.resize(sel_structures.size()-1);
    if( enemies_selected ) {
        enemies_selected = false;
    } else if( selstruct->canAttack() ) {
        --numattacking;
    }
}

void Selection::clearSelection() {
    Uint32 i;

    numinfantry = 0;
    numattacking = 0;
    for( i = 0; i < sel_units.size(); i++ ) {
        sel_units[i]->unSelect();
        sel_units[i]->unrefer();
    }
    for( i = 0; i < sel_structures.size(); i++ ) {
        sel_structures[i]->unSelect();
        sel_structures[i]->unSelect();
    }
    sel_units.resize(0);
    sel_structures.resize(0);
    enemies_selected = false;
}

bool Selection::canAttack() {
    return !enemies_selected && (numattacking > 0);
}

bool Selection::canMove() {
    // comment out check against enemies_selected
    // to test heat seeking code
    return !enemies_selected && (sel_units.size() > 0);
}


void Selection::moveUnits(Uint16 stop) {
    Uint32 i;

    for( i = 0; i < sel_units.size(); i++ ) {
        //      pt = new Path(sel_units[i]->getPos(), stop, costmatrix);
        //      if( pt != NULL )
        //	printf("Found a path\n");
        if( sel_units[i]->isAlive() ) {
            sel_units[i]->move(stop);
        }
    }

}

void Selection::attackUnit(Unit *target) {
    Uint32 i;

    for( i = 0; i < sel_units.size(); i++ ) {
        //      pt = new Path(sel_units[i]->getPos(), stop, costmatrix);
        //      if( pt != NULL )
        //	printf("Found a path\n");
        if( sel_units[i]->isAlive() && sel_units[i]->canAttack() ) {
            sel_units[i]->attackUnit(target);
        }
    }
    for( i = 0; i < sel_structures.size(); ++i) {
        /* add: when power checking is done, check if structure selected
         * can work given the player's power levels */
        if( sel_structures[i]->isAlive() && sel_structures[i]->canAttack() ) {
            sel_structures[i]->attackUnit(target);
        }
    }
}

void Selection::attackStructure(Structure *target) {
    Uint32 i;

    for( i = 0; i < sel_units.size(); i++ ) {
        if( sel_units[i]->isAlive() && ((UnitType *)sel_units[i]->getType())->getWeapon() != NULL &&
                (!target->isWall() || ((UnitType*)sel_units[i]->getType())->getWeapon()->getWall())) {
            sel_units[i]->attackStructure(target);
        }
    }
    for( i = 0; i < sel_structures.size(); ++i) {
        /* add: when power checking is done, check if structure selected
         * can work given the player's power levels */
        if( sel_structures[i]->isAlive() && sel_structures[i]->canAttack() &&
                (!target->isWall() || ((StructureType*)sel_structures[i]->getType())->getWeapon()->getWall())) {
            sel_structures[i]->attackStructure(target);
        }
    }
}

void Selection::checkSelection() {
    Uint16 i;

    for( i = 0; i < sel_units.size(); i++ ) {
        if( !sel_units[i]->isAlive() ) {
            removeUnit(sel_units[i]);
            i--;
        }
    }
    for( i = 0; i < sel_structures.size(); i++ ) {
        if( !sel_structures[i]->isAlive() ) {
            removeStructure(sel_structures[i]);
            i--;
        }
    }
}

Unit* Selection::getRandomUnit(void) {
    Uint8 rnd,sze;
    sze = sel_units.size();
    if (sze > 0) {
        rnd = (int) ((double)sze*rand()/(RAND_MAX+1.0));
        return sel_units[rnd];
    } else {
        return NULL;
    }
}

bool Selection::getWall() {
    Uint16 i;
    if (sel_units.size() == 0 && sel_structures.size() == 0)
        return false;
    for (i=0;i<sel_units.size();++i) {
        if ((((UnitType*)sel_units[i]->getType())->getWeapon() != NULL)&&(((UnitType*)sel_units[i]->getType())->getWeapon()->getWall())) {
            return true;
        }
    }
    for (i=0;i<sel_structures.size();++i) {
        if ((((StructureType*)sel_structures[i]->getType())->getWeapon() != NULL)&&(((StructureType*)sel_structures[i]->getType())->getWeapon()->getWall())) {
            return true;
        }
    }
    return false;
}

