/* Command-line aud player
 * used to debug the aud decoding and playing code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"
#include "inifile.h"
#include "soundengine.h"
#include "vfs.h"

void usage(char* biname) {
    printf("Usage: %s <audname.aud>\n",biname);
}

int main(int argc, char* argv[]) {
    SoundEngine* snd;

    if (argc < 2) {
        usage(argv[0]);
        exit(1);
    }
    if ((strcasecmp(argv[1],"-h")==0)||(strcasecmp(argv[1],"-help")==0)||
            (strcasecmp(argv[1],"--help")==0)) {
        usage(argv[0]);
        exit(1);
    }
    VFS_Init();
    if( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_NOPARACHUTE) < 0) {
        fprintf( stderr, "Couldn't initialize SDL: %s\n", SDL_GetError() );
        exit(1);
    }
    try {
        snd = new SoundEngine(false);
    } catch( SoundError *e ) {
        delete e;
        SDL_Quit();
        exit(1);
    }
    SDL_LockAudio();
    try {
        snd->queueSound(argv[1]);
    } catch( SoundError *e ) {
        fprintf(stderr,"File not found! %s\n",argv[1]);
        delete e;
        SDL_Quit();
        exit(1);
    }
    SDL_UnlockAudio();
    fprintf(stderr,"hit break to stop\n");
    while (1) {
        // TODO: some form of text based ui (r to restart etc.)
        SDL_Delay(10);
    }
    VFS_Destroy();
    SDL_Quit();
    exit(0);
}
